#include <LiquidCrystal.h>
#include <Time.h>
#include <math.h>
#include <EEPROM.h>
#include <Servo.h>

//
// GENERAL FUNCS
//
void btn_wait(int x){
  tone(13,4186,50);
  // wait for release of button before changing
  while (x > 0){
    x = analogRead(0);
    x = map(x,0,1023,0,10);
  }
}

//
// TIMER RELATED
//
unsigned long heater_tick = millis();
int time_mode = 0;
int pause_time[]={0,0,0};
int alarm_time[]={0,3,30};
boolean timer_paused = HIGH;

void reset_clock(){
  setTime(0,0,0,1,1,2011);
}

//
// THERMO RELATED
//
int target_temp = 40;
int actual_temp;
int tank_temp;

boolean heater_enabled = HIGH;
boolean use_celcius = HIGH;

double thermistor(int RawADC) {
  return int((5.0 * RawADC * 100.0)/1024.0);
}

//
// AGITATION RELATED
//
Servo ag_servo;
boolean ag_enabled = HIGH;
int ag_interval = 15;
unsigned long ag_tick = millis();
unsigned long ag_tock = millis();
boolean ag_servo_toggle = LOW;

//
// DISPLAY RELATED
//
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
int page_num = 3;
int last_page = 0;
int btn = 0;
unsigned long display_tick = millis();

byte icon_heat[8] = {
  B10001,
  B10001,
  B11111,
  B10001,
  B10001,
  B00000,
  B11111,

};

byte icon_agitate[8] = {
  B01110,
  B10001,
  B11111,
  B10001,
  B10001,
  B00000,
  B11111,
};

byte icon_time[8] = {
  B11111,
  B10101,
  B00100,
  B00100,
  B01110,
  B00000,
  B11111
};

byte icon_left[8] = {
  B11100,
  B11000,
  B11000,
  B11000,
  B11000,
  B11000,
  B11111
};

byte icon_right[8] = {
  B00111,
  B00011,
  B00011,
  B00011,
  B00011,
  B00011,
  B11111
};
byte icon_blank[8] = {
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B11111
};

String clock(String h, String m, String s){
  // RETURNS A STRING WITH NICELY
  // NICELY FORMATTED CLOCK
  if (h.length() ==1){
  h = "0"+h;
  }
  if (m.length() ==1){
  m = "0"+m;
  }
  if (s.length() ==1){
  s = "0"+s;
  }  
  return h+":"+m+":"+s;
}

void update_display(){
  // don't update display too often
  if (millis() < display_tick+500){
    return;
  }
  display_tick = millis();
    
  // UPDATE DISPLAY DEPENDING ON PAGE NUM
  if (page_num != last_page){  
    lcd.clear();
    last_page = page_num;
  }
  
  switch (page_num){
    case 0:
      // temperature control
      lcd.setCursor(0, 0);
        if (use_celcius == HIGH){
        lcd.print("T:");
        lcd.print(actual_temp);
        lcd.print("C  ");
        lcd.setCursor(8, 0);
        lcd.print("Max:");
        lcd.print(target_temp);
        lcd.print("C  ");
      } else {
        lcd.print("T:");
        lcd.print(int((actual_temp*9.0)/5.0 + 32.0));
        lcd.print("F  ");
        lcd.setCursor(8, 0);
        lcd.print("Max:");
        lcd.print(int((target_temp*9.0)/5.0 + 32.0));
        lcd.print("F  ");
      }
      
      lcd.setCursor(0, 1);
      lcd.print("Heater: ");
      if (heater_enabled == HIGH){
        lcd.print("ON ");
      } else {
        lcd.print("OFF");
      }

      
      break;
      
    case 1:
      // timer control
      lcd.setCursor(0, 0);
      lcd.print("Alarm:  ");
      lcd.print(clock(alarm_time[0],alarm_time[1],alarm_time[2]));
      lcd.setCursor(0, 1);
      lcd.print("Actual: ");
      if (timer_paused == LOW){
        lcd.print(clock(hour(),minute(),second()));
      } else{
        lcd.print(clock(pause_time[0],pause_time[1],pause_time[2]));      
      }
      break;
      
    case 2:
      // Agitation control
      lcd.setCursor(0, 0);
      lcd.print("Agitation: ");
      if (ag_enabled == HIGH){
        lcd.print("ON ");
      } else {
        lcd.print("OFF");
      }
      lcd.setCursor(0, 1);
      lcd.print("Interval:  ");
      lcd.print(ag_interval);
      lcd.print("s  ");      
      break;
      
    case 3:
      // overview screen
      lcd.setCursor(0, 0);
      lcd.write(0);
      if (heater_enabled == HIGH){
        lcd.write(5);
      } else {
        lcd.write(2);
      }
      if (ag_enabled == HIGH){
        lcd.write(4);
      } else {
        lcd.write(2);
      }
      if (timer_paused == LOW){
        lcd.write(3);
      } else {
        lcd.write(2);
      }
      lcd.write(1);


      lcd.setCursor(0, 1);
      if (use_celcius == HIGH){
        lcd.print(actual_temp);
        lcd.print("C  ");
      } else {
        lcd.print(int((actual_temp*9.0)/5.0 + 32.0));
        lcd.print("F  ");
      }
      
      lcd.setCursor(8, 0);
      lcd.print(clock(alarm_time[0],alarm_time[1],alarm_time[2]));
  
      lcd.setCursor(8, 1);
      if (timer_paused == LOW){
        lcd.print(clock(hour(),minute(),second()));
      } else{
        lcd.print(clock(pause_time[0],pause_time[1],pause_time[2]));      
      }
      break;
  }
  
}

//
// SETUP HERE
//

void setup()
{
  // START THE ENGINES
  lcd.createChar(0, icon_left);
  lcd.createChar(1, icon_right);
  lcd.createChar(2, icon_blank);
  lcd.createChar(3, icon_time);
  lcd.createChar(4, icon_agitate);
  lcd.createChar(5, icon_heat);
  
  lcd.begin(16,2);
  lcd.setCursor(0, 0);
  lcd.print("  MONKEYSAILOR  ");
  lcd.setCursor(0, 1);
  lcd.print("   PHOTO  LAB   ");
  reset_clock();

  // SET THE PIN MODES
  pinMode(1, OUTPUT); // THERMISTOR POWER
  pinMode(6, OUTPUT); // HEATER POWER
  pinMode(7, OUTPUT); // TANK POWER
  ag_servo.attach(8); // SERVO
  
  // PRELOAD FROM EEPROM
  target_temp = int(EEPROM.read(0));
  alarm_time[0] = int(EEPROM.read(1));
  alarm_time[1] = int(EEPROM.read(2));
  alarm_time[2] = int(EEPROM.read(3));  
  ag_enabled = boolean(EEPROM.read(4));
  ag_interval = int(EEPROM.read(5));
  heater_enabled = boolean(EEPROM.read(6));
  use_celcius = boolean(EEPROM.read(7));
  
  // PLAY WELCOME CHIME
  for (int i=512;i<1024;i=i+25){
      tone(13,i,20);
      delay(20);
      tone(13,1024-i,20);
      delay(20);
  }

  for (int i=1024;i<1576;i=i+25){
      tone(13,i,20);
      delay(20);
      tone(13,1576-i,20);
      delay(20);
  }
  
  for (int i=512;i<1024;i=i+25){
      tone(13,i,20);
      delay(20);
      tone(13,1024-i,20);
      delay(20);
  }  
  delay(3000);
  lcd.clear();
}


//
// MAIN LOOP HERE
//

void loop() {
  
  // GET THE SENSOR READINGS
  if (millis() > heater_tick+2000){
    heater_tick = millis();
    // USE PIN1 TO POWER THERMISTOR TO PREVENT SELF HEATING
    digitalWrite(1,HIGH);
    delay(200);
    actual_temp = thermistor(analogRead(5));
    tank_temp = thermistor(analogRead(4));
    digitalWrite(1,LOW);
  }
  
  // CHECK THE TEMP
  if (heater_enabled == HIGH){
    // BOTTTLE TEMP
    if (actual_temp > target_temp){
      digitalWrite(6,LOW);
    } else {
      digitalWrite(6,HIGH);
    }
    
    // PATTERSON TANK WARMER TEMP
    if (tank_temp > target_temp){
      digitalWrite(7,LOW);
    } else {
      digitalWrite(7,HIGH);
    }
    
  } else {
    digitalWrite(6,LOW);
    digitalWrite(7,LOW);
  }
  
  // CHECK AGITATOR
  if (ag_enabled == HIGH && timer_paused == LOW){
    if (millis() > (ag_tick+(ag_interval*1000))){
      // if the tock is less than 15 seconds, shuffle the servo
      if (millis() < ag_tock+15000){
        if (ag_servo_toggle==LOW){
          ag_servo.write(5);
          ag_servo_toggle = HIGH;
         delay(500); 
        }else{
          ag_servo.write(175);
          ag_servo_toggle = LOW; 
          delay(500);
        }
      } else {
        ag_tick = millis();
        ag_tock = millis();
      }
    } else {
        ag_tock = millis();
    }

  }
  
  // CHECK FOR ALARM TIME
  if (timer_paused==LOW && (alarm_time[0] > 0 || alarm_time[1] > 0 || alarm_time[2] > 0)){
    if (alarm_time[0] == hour() && alarm_time[1] == minute() && alarm_time[2] == second()){
      for ( int i = 0; i < 10; i++){
        tone(13, 262,200);
        delay(200);
        tone(13, 523,200);
        delay(200);
      } //endfor
    }
  }

  // GET BUTTON STATUS
  btn = analogRead(0);
  btn = map(btn,0,1023,0,10);
  
  // PAGE INDEPENDENT BUTTON COMMANDS
  switch (btn){
    case 1:
      // CHANGE PAGE
      if (page_num <3){
        page_num = page_num+1;
      } else {
        page_num = 0;
      }
      // wait for release of button before changing
      btn_wait(btn);     
    break;
    
    case 7:
      // RESTORE DEFAULTS FROM EEPROM
      target_temp = int(EEPROM.read(0));
      alarm_time[0] = int(EEPROM.read(1));
      alarm_time[1] = int(EEPROM.read(2));
      alarm_time[2] = int(EEPROM.read(3));  
      ag_enabled = boolean(EEPROM.read(4));
      ag_interval = int(EEPROM.read(5));
      heater_enabled = boolean(EEPROM.read(6));
      use_celcius = boolean(EEPROM.read(7));      
      btn_wait(btn);       
      break;
    
    case 8:
       // SAVE CHANGES
      EEPROM.write(0, byte(target_temp));
      EEPROM.write(1, byte(alarm_time[0]));
      EEPROM.write(2, byte(alarm_time[1]));
      EEPROM.write(3, byte(alarm_time[2]));
      EEPROM.write(4, byte(ag_enabled));
      EEPROM.write(5, byte(ag_interval));
      EEPROM.write(6, byte(heater_enabled));
      EEPROM.write(7, byte(use_celcius));      
      btn_wait(btn);       
      break;
  
  }

  // PAGE DEPENDENT BUTTON COMMANDS
  switch (page_num){
    case 0:
      // temperature control
      switch(btn){
        case 2:
          //inc temp
          if (target_temp < 60){
            target_temp = target_temp+1;          
          }
          delay(200);      
          break;
        
        case 3:
          //dec temp
          if (target_temp > 20){
            target_temp = target_temp-1;          
          }        
          delay(200);
          break;
        case 4:
          // celcius or farenheit
          if (use_celcius == HIGH){
            use_celcius = LOW;
          } else {
            use_celcius = HIGH;  
          }
          btn_wait(btn);
          break;
      
        case 6:
          if (heater_enabled == HIGH){
            heater_enabled = LOW;
          } else {
            heater_enabled = HIGH;
          }
          btn_wait(btn);
          break;          

      }
      break; 
      
    case 1:
      // timer control
      switch(btn){
        case 2:
          // inc
          alarm_time[time_mode]=alarm_time[time_mode]+1;
          if (alarm_time[time_mode]>59){
            alarm_time[time_mode]=0;
          }
          delay(200);
          break;
          
        case 3:
          // dec
          alarm_time[time_mode]=alarm_time[time_mode]-1;
          if (alarm_time[time_mode]<0){
            alarm_time[time_mode]=59;
          }
          delay(200);
          break;
          
        case 4:
          // toggle h/m/s
          if (time_mode <2){
            time_mode = time_mode+1;
          } else {
            time_mode = 0;
          }
          btn_wait(btn);
          break;
          
        case 5:
          // reset
          btn_wait(btn);
          pause_time[0] = 0;
          pause_time[1] = 0;
          pause_time[2] = 0;
          reset_clock();
          timer_paused = HIGH;
          break;
          
        case 6:
          // stop-start
          btn_wait(btn);
          if (timer_paused == HIGH){
            // restart
            setTime(pause_time[0], pause_time[1], pause_time[2],1,1,2011);
            timer_paused = LOW;
            break; 
            
          } else {
            //stop
            timer_paused = HIGH;
            pause_time[0] = hour();
            pause_time[1] = minute();
            pause_time[2] = second();

            break; 
          }   
      }
      break;
      
    case 2:
      // Agitation control
      switch(btn){
        case 2:
          // inc
          ag_interval = ag_interval+1;
          if (ag_interval>60){
            ag_interval = 15;
          }
          delay(200);
          break;
          
        case 3:
          // dec
          ag_interval = ag_interval-1;;
          if (ag_interval<15){
            ag_interval = 60;
          }
          delay(200);
          break;
      
        case 4:
          if (ag_enabled == HIGH){
            ag_enabled = LOW;
          } else {
            ag_enabled = HIGH;
          }
          btn_wait(btn);
          break;
          
        case 6:
          if (ag_servo_toggle==LOW){
            ag_servo.write(5);
            ag_servo_toggle = HIGH;
            delay(500); 
          }else{
            ag_servo.write(175);
            ag_servo_toggle = LOW; 
            delay(500);
          }
          break;
      }
 
      break;

    case 3:
      // Overview controls
      switch(btn){
        case 2:
        // heater enable-disable
          if (heater_enabled == HIGH){
            heater_enabled = LOW;
          } else {
            heater_enabled = HIGH;
          }
          btn_wait(btn);
          break;          

          
        case 3:
        // agitator enable-disable
          if (ag_enabled == HIGH){
            ag_enabled = LOW;
          } else {
            ag_enabled = HIGH;
          }
          btn_wait(btn);
          break;
      
         case 4:
           if (ag_servo_toggle==LOW){
             ag_servo.write(5);
             ag_servo_toggle = HIGH;
             delay(500); 
           }else{
             ag_servo.write(175);
             ag_servo_toggle = LOW; 
             delay(500);
           }
          break;

          
        case 5:
          // reset
          btn_wait(btn);
          pause_time[0] = 0;
          pause_time[1] = 0;
          pause_time[2] = 0;
          reset_clock();
          timer_paused = HIGH;
          break;
          
        case 6:
          // stop-start
          btn_wait(btn);
          if (timer_paused == HIGH){
            // restart
            setTime(pause_time[0], pause_time[1], pause_time[2],1,1,2011);
            timer_paused = LOW;
            break; 
            
          } else {
            //stop
            timer_paused = HIGH;
            pause_time[0] = hour();
            pause_time[1] = minute();
            pause_time[2] = second();

            break; 
          }   
      }
      break;    
  }
  
  // UPDATE THE DISPLAY
  update_display();
}

